/* Copyright 2020-2025, Michele Santullo
 * This file is part of memoserv.
 *
 * Memoserv is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Memoserv is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Memoserv.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "savegame_model.hpp"
#include "savegame_db.hpp"
#include <cassert>
#include <stdexcept>
#include <ciso646>

namespace duck {
SavegameModel::SavegameModel() = default;
SavegameModel::~SavegameModel() noexcept = default;

MemoryCardSavegameModel::MemoryCardSavegameModel (mc::psx::MemoryCard* memcard) :
	m_mc(memcard)
{
	assert(m_mc);
}

SavegameModel::iterator MemoryCardSavegameModel::begin() {
	return {m_mc->begin()};
}

SavegameModel::iterator MemoryCardSavegameModel::end() {
	return {m_mc->end()};
}

std::size_t MemoryCardSavegameModel::block_count() const {
	return m_mc->size();
}

bool MemoryCardSavegameModel::has_toc() const {
	return true;
}

mc::psx::Block MemoryCardSavegameModel::toc() {
	return m_mc->toc_block();
}

mc::psx::ConstBlock MemoryCardSavegameModel::toc() const {
	return m_mc->toc_block();
}

mc::psx::Block MemoryCardSavegameModel::block (std::size_t index) {
	return (*m_mc)[index];
}

mc::psx::ConstBlock MemoryCardSavegameModel::block (std::size_t index) const {
	return (*m_mc)[index];
}

DbSavegameModel::DbSavegameModel (SavegameDb* db) :
	m_db(db)
{
	assert(m_db);
}

SavegameModel::iterator DbSavegameModel::begin() {
	return {m_db->begin()};
}

SavegameModel::iterator DbSavegameModel::end() {
	return {m_db->end()};
}

std::size_t DbSavegameModel::block_count() const {
	return m_db->block_count();
}

bool DbSavegameModel::has_toc() const {
	return false;
}

mc::psx::Block DbSavegameModel::toc() {
	throw std::runtime_error("TOC not available in SavegameDb");
}

mc::psx::ConstBlock DbSavegameModel::toc() const {
	throw std::runtime_error("TOC not available in SavegameDb");
}

mc::psx::Block DbSavegameModel::block (std::size_t index) {
	auto it = begin();
	std::advance(it, index);
	return *it;
}

mc::psx::ConstBlock DbSavegameModel::block (std::size_t index) const {
}

} //namespace duck
