/* Copyright 2020-2025, Michele Santullo
 * This file is part of memoserv.
 *
 * Memoserv is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Memoserv is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Memoserv.  If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include "memcard/memorycard.hpp"
#include "any_iterator.hpp"
#include <cstddef>

namespace mc::psx {
template <bool> class BasicBlock;
};

namespace duck {
class SavegameDb;

class SavegameModel {
public:
	typedef liph::any_forward_iterator<
		mc::psx::BasicBlock<false>,
		mc::psx::BasicBlock<false>,
		mc::psx::BasicBlock<false>
	> iterator;

	SavegameModel();
	virtual ~SavegameModel() noexcept;

	std::size_t size() const { return this->block_count(); }

	virtual iterator begin() = 0;
	virtual iterator end() = 0;
	virtual std::size_t block_count() const = 0;
	virtual bool has_toc() const = 0;
	virtual mc::psx::Block toc() = 0;
	virtual mc::psx::ConstBlock toc() const = 0;
	virtual mc::psx::Block block (std::size_t index) = 0;
	virtual mc::psx::ConstBlock block (std::size_t index) const = 0;
private:
};

class MemoryCardSavegameModel : public SavegameModel {
public:
	explicit MemoryCardSavegameModel (mc::psx::MemoryCard* memcard);
	virtual ~MemoryCardSavegameModel() noexcept = default;

	SavegameModel::iterator begin() override;
	SavegameModel::iterator end() override;
	std::size_t block_count() const override;
	bool has_toc() const override;
	mc::psx::Block toc() override;
	mc::psx::ConstBlock toc() const override;
	mc::psx::Block block (std::size_t index) override;
	mc::psx::ConstBlock block (std::size_t index) const override;

private:
	mc::psx::MemoryCard* m_mc;
};

class DbSavegameModel : public SavegameModel {
public:
	explicit DbSavegameModel (SavegameDb* db);
	virtual ~DbSavegameModel() noexcept = default;

	SavegameModel::iterator begin() override;
	SavegameModel::iterator end() override;
	std::size_t block_count() const override;
	bool has_toc() const override;
	mc::psx::Block toc() override;
	mc::psx::ConstBlock toc() const override;
	mc::psx::Block block (std::size_t index) override;
	mc::psx::ConstBlock block (std::size_t index) const override;

private:
	SavegameDb* m_db;
};
} //namespace duck
