/* Copyright 2020-2025, Michele Santullo
 * This file is part of memoserv.
 *
 * Memoserv is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Memoserv is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Memoserv.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "memcard/alignment.hpp"
#include "memcard/block.hpp"
#include "duckhandy/packed_pointer.hpp"

namespace mc::psx {
std::uint8_t* to_block_aligned (std::uint8_t* ptr, std::size_t index, std::ptrdiff_t base_offs) {
	return reinterpret_cast<std::uint8_t*>(
		(reinterpret_cast<std::uintptr_t>(ptr) + (Block::DataAlignment-1)) & ~(Block::DataAlignment-1)
	) + base_offs + index * Block::BlockByteSize;
}

const std::uint8_t* to_block_aligned (const std::uint8_t* ptr, std::size_t index, std::ptrdiff_t base_offs) {
	return const_cast<const std::uint8_t*>(
		to_block_aligned(const_cast<std::uint8_t*>(ptr), index, base_offs)
	);
}

bool is_block_aligned (const std::uint8_t* ptr) {
	return (0 == reinterpret_cast<std::uintptr_t>(ptr) % Block::DataAlignment);
}

template <typename T, std::size_t A>
bool is_block_aligned (const dhandy::PackedPointer<T, std::size_t, A>& ptr) {
	return (0 == static_cast<std::uintptr_t>(ptr) % Block::DataAlignment);
}

template bool is_block_aligned<std::uint8_t, Block::DataAlignment> (
	const dhandy::PackedPointer<std::uint8_t, std::size_t, Block::DataAlignment>&
);
template bool is_block_aligned<const std::uint8_t, Block::DataAlignment> (
	const dhandy::PackedPointer<const std::uint8_t, std::size_t, Block::DataAlignment>&
);

std::unique_ptr<std::uint8_t[]> make_unique_for_block_aligned(std::size_t size) {
	return std::make_unique<std::uint8_t[]>(size + Block::DataAlignment - 1);
}
} //namespace mc::psx
